from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{prefix}} {{last_name}}",
        "{{first_name_female}} {{last_name}}-{{last_name}}",
        "{{first_name_female}}-{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{prefix}} {{last_name}}",
    )

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{prefix}} {{last_name}}",
        "{{first_name_male}} {{last_name}}-{{last_name}}",
        "{{first_name_male}}-{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{prefix}} {{last_name}}",
    )

    formats = formats_male + formats_female

    first_names_male = (
        "Adrien",
        "Aimé",
        "Alain",
        "Alexandre",
        "Alfred",
        "Alphonse",
        "André",
        "Antoine",
        "Arthur",
        "Auguste",
        "Augustin",
        "Benjamin",
        "Benoît",
        "Bernard",
        "Bertrand",
        "Charles",
        "Christophe",
        "Daniel",
        "David",
        "Denis",
        "Édouard",
        "Émile",
        "Emmanuel",
        "Éric",
        "Étienne",
        "Eugène",
        "François",
        "Franck",
        "Frédéric",
        "Gabriel",
        "Georges",
        "Gérard",
        "Gilbert",
        "Gilles",
        "Grégoire",
        "Guillaume",
        "Guy",
        "William",
        "Henri",
        "Honoré",
        "Hugues",
        "Isaac",
        "Jacques",
        "Jean",
        "Jérôme",
        "Joseph",
        "Jules",
        "Julien",
        "Laurent",
        "Léon",
        "Louis",
        "Luc",
        "Lucas",
        "Marc",
        "Marcel",
        "Martin",
        "Matthieu",
        "Maurice",
        "Michel",
        "Nicolas",
        "Noël",
        "Olivier",
        "Patrick",
        "Paul",
        "Philippe",
        "Pierre",
        "Raymond",
        "Rémy",
        "René",
        "Richard",
        "Robert",
        "Roger",
        "Roland",
        "Sébastien",
        "Stéphane",
        "Théodore",
        "Théophile",
        "Thibaut",
        "Thibault",
        "Thierry",
        "Thomas",
        "Timothée",
        "Tristan",
        "Victor",
        "Vincent",
        "Xavier",
        "Yves",
        "Zacharie",
    )

    first_names_female = (
        "Adélaïde",
        "Adèle",
        "Adrienne",
        "Agathe",
        "Agnès",
        "Aimée",
        "Alexandrie",
        "Alix",
        "Alexandria",
        "Alex",
        "Alice",
        "Amélie",
        "Anaïs",
        "Anastasie",
        "Andrée",
        "Anne",
        "Anouk",
        "Antoinette",
        "Arnaude",
        "Astrid",
        "Audrey",
        "Aurélie",
        "Aurore",
        "Bernadette",
        "Brigitte",
        "Capucine",
        "Caroline",
        "Catherine",
        "Cécile",
        "Céline",
        "Célina",
        "Chantal",
        "Charlotte",
        "Christelle",
        "Christiane",
        "Christine",
        "Claire",
        "Claudine",
        "Clémence",
        "Colette",
        "Constance",
        "Corinne",
        "Danielle",
        "Denise",
        "Diane",
        "Dorothée",
        "Édith",
        "Éléonore",
        "Élisabeth",
        "Élise",
        "Élodie",
        "Émilie",
        "Emmanuelle",
        "Françoise",
        "Frédérique",
        "Gabrielle",
        "Geneviève",
        "Hélène",
        "Henriette",
        "Hortense",
        "Inès",
        "Isabelle",
        "Jacqueline",
        "Jeanne",
        "Jeannine",
        "Joséphine",
        "Josette",
        "Julie",
        "Juliette",
        "Laetitia",
        "Laure",
        "Laurence",
        "Lorraine",
        "Louise",
        "Luce",
        "Lucie",
        "Lucy",
        "Madeleine",
        "Manon",
        "Marcelle",
        "Margaux",
        "Margaud",
        "Margot",
        "Marguerite",
        "Margot",
        "Margaret",
        "Maggie",
        "Marianne",
        "Marie",
        "Marine",
        "Marthe",
        "Martine",
        "Maryse",
        "Mathilde",
        "Michèle",
        "Michelle",
        "Michelle",
        "Monique",
        "Nathalie",
        "Nath",
        "Nathalie",
        "Nicole",
        "Noémi",
        "Océane",
        "Odette",
        "Olivie",
        "Patricia",
        "Paulette",
        "Pauline",
        "Pénélope",
        "Philippine",
        "Renée",
        "Sabine",
        "Simone",
        "Sophie",
        "Stéphanie",
        "Susanne",
        "Suzanne",
        "Susan",
        "Suzanne",
        "Sylvie",
        "Thérèse",
        "Valentine",
        "Valérie",
        "Véronique",
        "Victoire",
        "Virginie",
        "Zoé",
        "Camille",
        "Claude",
        "Dominique",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Martin",
        "Bernard",
        "Thomas",
        "Robert",
        "Petit",
        "Dubois",
        "Richard",
        "Garcia",
        "Durand",
        "Moreau",
        "Lefebvre",
        "Simon",
        "Laurent",
        "Michel",
        "Leroy",
        "Martinez",
        "David",
        "Fontaine",
        "Da Silva",
        "Morel",
        "Fournier",
        "Dupont",
        "Bertrand",
        "Lambert",
        "Rousseau",
        "Girard",
        "Roux",
        "Vincent",
        "Lefèvre",
        "Boyer",
        "Lopez",
        "Bonnet",
        "Andre",
        "François",
        "Mercier",
        "Muller",
        "Guérin",
        "Legrand",
        "Sanchez",
        "Garnier",
        "Chevalier",
        "Faure",
        "Perez",
        "Clément",
        "Fernandez",
        "Blanc",
        "Robin",
        "Morin",
        "Gauthier",
        "Pereira",
        "Perrin",
        "Roussel",
        "Henry",
        "Duval",
        "Gautier",
        "Nicolas",
        "Masson",
        "Marie",
        "Noël",
        "Ferreira",
        "Lemaire",
        "Mathieu",
        "Rivière",
        "Denis",
        "Marchand",
        "Rodriguez",
        "Dumont",
        "Payet",
        "Lucas",
        "Dufour",
        "Dos Santos",
        "Joly",
        "Blanchard",
        "Meunier",
        "Rodrigues",
        "Caron",
        "Gérard",
        "Fernandes",
        "Brunet",
        "Meyer",
        "Barbier",
        "Leroux",
        "Renard",
        "Goncalves",
        "Gaillard",
        "Brun",
        "Roy",
        "Picard",
        "Giraud",
        "Roger",
        "Schmitt",
        "Colin",
        "Arnaud",
        "Vidal",
        "Gonzalez",
        "Lemoine",
        "Roche",
        "Aubert",
        "Olivier",
        "Leclercq",
        "Pierre",
        "Philippe",
        "Bourgeois",
        "Renaud",
        "Martins",
        "Leclerc",
        "Guillaume",
        "Lacroix",
        "Lecomte",
        "Benoit",
        "Fabre",
        "Carpentier",
        "Vasseur",
        "Louis",
        "Hubert",
        "Jean",
        "Dumas",
        "Rolland",
        "Grondin",
        "Rey",
        "Huet",
        "Gomez",
        "Dupuis",
        "Guillot",
        "Berger",
        "Moulin",
        "Hoarau",
        "Menard",
        "Deschamps",
        "Fleury",
        "Adam",
        "Boucher",
        "Poirier",
        "Bertin",
        "Charles",
        "Aubry",
        "Da Costa",
        "Royer",
        "Dupuy",
        "Maillard",
        "Paris",
        "Baron",
        "Lopes",
        "Guyot",
        "Carre",
        "Jacquet",
        "Renault",
        "Hervé",
        "Charpentier",
        "Klein",
        "Cousin",
        "Collet",
        "Léger",
        "Ribeiro",
        "Hernandez",
        "Bailly",
        "Schneider",
        "Le Gall",
        "Ruiz",
        "Langlois",
        "Bouvier",
        "Gomes",
        "Prévost",
        "Julien",
        "Lebrun",
        "Breton",
        "Germain",
        "Millet",
        "Boulanger",
        "Rémy",
        "Le Roux",
        "Daniel",
        "Marques",
        "Maillot",
        "Leblanc",
        "Le Goff",
        "Barre",
        "Perrot",
        "Lévêque",
        "Marty",
        "Benard",
        "Monnier",
        "Hamon",
        "Pelletier",
        "Alves",
        "Étienne",
        "Marchal",
        "Poulain",
        "Tessier",
        "Lemaître",
        "Guichard",
        "Besson",
        "Mallet",
        "Hoareau",
        "Gillet",
        "Weber",
        "Jacob",
        "Collin",
        "Chevallier",
        "Perrier",
        "Michaud",
        "Carlier",
        "Delaunay",
        "Chauvin",
        "Alexandre",
        "Maréchal",
        "Antoine",
        "Lebon",
        "Cordier",
        "Lejeune",
        "Bouchet",
        "Pasquier",
        "Legros",
        "Delattre",
        "Humbert",
        "De Oliveira",
        "Briand",
        "Lamy",
        "Launay",
        "Gilbert",
        "Perret",
        "Lesage",
        "Gay",
        "Nguyen",
        "Navarro",
        "Besnard",
        "Pichon",
        "Hebert",
        "Cohen",
        "Pons",
        "Lebreton",
        "Sauvage",
        "De Sousa",
        "Pineau",
        "Albert",
        "Jacques",
        "Pinto",
        "Barthelemy",
        "Turpin",
        "Bigot",
        "Lelièvre",
        "Georges",
        "Reynaud",
        "Ollivier",
        "Martel",
        "Voisin",
        "Leduc",
        "Guillet",
        "Vallée",
        "Coulon",
        "Camus",
        "Marin",
        "Teixeira",
        "Costa",
        "Mahe",
        "Didier",
        "Charrier",
        "Gaudin",
        "Bodin",
        "Guillou",
        "Grégoire",
        "Gros",
        "Blanchet",
        "Buisson",
        "Blondel",
        "Paul",
        "Dijoux",
        "Barbe",
        "Hardy",
        "Laine",
        "Evrard",
        "Laporte",
        "Rossi",
        "Joubert",
        "Regnier",
        "Tanguy",
        "Gimenez",
        "Allard",
        "Devaux",
        "Morvan",
        "Lévy",
        "Dias",
        "Courtois",
        "Lenoir",
        "Berthelot",
        "Pascal",
        "Vaillant",
        "Guilbert",
        "Thibault",
        "Moreno",
        "Duhamel",
        "Colas",
        "Masse",
        "Baudry",
        "Bruneau",
        "Verdier",
        "Delorme",
        "Blin",
        "Guillon",
        "Mary",
        "Coste",
        "Pruvost",
        "Maury",
        "Allain",
        "Valentin",
        "Godard",
        "Joseph",
        "Brunel",
        "Marion",
        "Texier",
        "Seguin",
        "Raynaud",
        "Bourdon",
        "Raymond",
        "Bonneau",
        "Chauvet",
        "Maurice",
        "Legendre",
        "Loiseau",
        "Ferrand",
        "Toussaint",
        "Techer",
        "Lombard",
        "Lefort",
        "Couturier",
        "Bousquet",
        "Diaz",
        "Riou",
        "Clerc",
        "Weiss",
        "Imbert",
        "Jourdan",
        "Delahaye",
        "Gilles",
        "Guibert",
        "Bègue",
        "Descamps",
        "Delmas",
        "Peltier",
        "Dupré",
        "Chartier",
        "Martineau",
        "Laroche",
        "Leconte",
        "Maillet",
        "Parent",
        "Labbé",
        "Potier",
        "Bazin",
        "Normand",
        "Pottier",
        "Torres",
        "Lagarde",
        "Blot",
        "Jacquot",
        "Lemonnier",
        "Grenier",
        "Rocher",
        "Bonnin",
        "Boutin",
        "Fischer",
        "Munoz",
        "Neveu",
        "Lacombe",
        "Mendès",
        "Delannoy",
        "Auger",
        "Wagner",
        "Fouquet",
        "Mace",
        "Ramos",
        "Pages",
        "Petitjean",
        "Chauveau",
        "Foucher",
        "Peron",
        "Guyon",
        "Gallet",
        "Rousset",
        "Traore",
        "Bernier",
        "Vallet",
        "Letellier",
        "Bouvet",
        "Hamel",
        "Chrétien",
        "Faivre",
        "Boulay",
        "Thierry",
        "Samson",
        "Ledoux",
        "Salmon",
        "Gosselin",
        "Lecoq",
        "Pires",
        "Leleu",
        "Becker",
        "Diallo",
        "Merle",
        "Valette",
    )

    prefixes = ("de", "de la", "Le", "du")



import random

from datetime import date, datetime
from typing import Optional

from ....typing import SexLiteral
from .. import Provider as SsnProvider


def select_gender(gender: SexLiteral) -> int:
    """Choose an even number for Female and odd number for Male."""
    gender = 0 if gender.lower() == "f" else 1
    return random.choice(range(gender, 10, 2))


def calculate_day_count(birthday: date) -> int:
    """Calculate the day count from reference date '31 December 1899'."""
    ref_date = date(1899, 12, 31)
    return (birthday - ref_date).days


def calculate_check_sum(val: str) -> int:
    """Calculate checksum using INN calculation method."""
    weights = [-1, 5, 7, 9, 4, 6, 10, 5, 7]
    checksum = sum(int(v) * w for v, w in zip(val, weights))

    return checksum % 11 % 10


class Provider(SsnProvider):
    def ssn(self, birthday: Optional[str] = None, gender: Optional[SexLiteral] = None) -> str:
        """
        Ukrainian "Реєстраційний номер облікової картки платника податків"
        also known as "Ідентифікаційний номер фізичної особи".
        @params: birthday: "DD-MM-YYYY" format, default random date
        @params: gender: "M" or "F" default: random gender

        :sample:
        :sample: birthday='22-06-1990', gender='F'
        """

        try:
            # generate day of birthday date object
            if birthday:
                dob = datetime.strptime(birthday, "%d-%m-%Y").date()
            else:
                dob = self.generator.date_object()
        except Exception:
            raise ValueError("Birthday format must be DD-MM-YYYY")

        if gender and gender not in ("M", "F"):
            raise ValueError('Gender must be "m" or "f" or None')

        day_count = calculate_day_count(dob)
        people_num = self.random_number(3, fix_len=True)
        gender_ = select_gender(gender) if gender else random.randint(0, 1)
        ssn_without_checksum = f"{day_count}{people_num}{gender_}"
        checksum = calculate_check_sum(ssn_without_checksum)
        return f"{ssn_without_checksum}{checksum}"



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``fr_FR`` locale."""

    all_colors = OrderedDict(
        (
            ("Noir", "#000000"),
            ("Gris mat", "#696969"),
            ("Gris", "#808080"),
            ("Gris foncé (Acier)", "#A9A9A9"),
            ("Gris argent", "#C0C0C0"),
            ("Gris clair", "#D3D3D3"),
            ("Gris gainsboro (Etain)", "#DCDCDC"),
            ("Blanc fumée", "#F5F5F5"),
            ("Blanc spectral", "#F8F8FF"),
            ("Blanc", "#FFFFFF"),
            ("Ivoire", "#FFFFF0"),
            ("Blanc floral", "#FFFAF0"),
            ("Blanc coquillage", "#FFF5EE"),
            ("Blanc lavande", "#FFF0F5"),
            ("Blanc dentelle", "#FDF5E6"),
            ("Blanc Lin", "#FAF0E6"),
            ("Rose brumeux", "#FFE4E1"),
            ("Rose", "#FFC0CB"),
            ("Rose clair", "#FFB6C1"),
            ("Rose Passion", "#FF69B4"),
            ("Rose profond", "#FF1493"),
            ("Violet pâle", "#DB7093"),
            ("Fushia (Magenta)", "#FF00FF"),
            ("Violet moyen", "#C71585"),
            ("Violet chardon", "#D8BFD8"),
            ("Prune", "#DDA0DD"),
            ("Violet", "#EE82EE"),
            ("Violet orchidée", "#DA70D6"),
            ("Violet orchidée moyen", "#BA55D3"),
            ("Violet orchidée foncé", "#9932CC"),
            ("Violet foncé", "#9400D3"),
            ("Bleu violet", "#8A2BE2"),
            ("Indigo", "#4B0082"),
            ("Bleu ardoise moyen", "#7B68EE"),
            ("Bleu ardoise", "#6A5ACD"),
            ("Bleu ardoise foncé", "#483D8B"),
            ("Pourpre moyen", "#9370DB"),
            ("Magenta foncé", "#8B008B"),
            ("Pourpre", "#800080"),
            ("Brun rosé", "#BC8F8F"),
            ("Corail clair", "#F08080"),
            ("Corail", "#FF7F50"),
            ("Tomate", "#FF6347"),
            ("Orangé", "#FF4500"),
            ("Rouge", "#FF0000"),
            ("Rouge cramoisi", "#DC143C"),
            ("Saumon clair", "#FFA07A"),
            ("Saumon Foncé", "#E9967A"),
            ("Saumon", "#FA8072"),
            ("Rouge Indien", "#CD5C5C"),
            ("Rouge brique", "#B22222"),
            ("Brun", "#A52A2A"),
            ("Rouge foncé", "#8B0000"),
            ("Bordeaux", "#800000"),
            ("Beige", "#F5F5DC"),
            ("Beige antique", "#FAEBD7"),
            ("Beige papaye", "#FFEFD5"),
            ("Amande", "#FFEBCD"),
            ("Bisque", "#FFE4C4"),
            ("Beige pêche", "#FFDAB9"),
            ("Beige mocassin", "#FFE4B5"),
            ("Jaune blanc navaro", "#FFDEAD"),
            ("Jaune blé", "#F5DEB3"),
            ("Brun bois rustique", "#DEB887"),
            ("Brun roux", "#D2B48C"),
            ("Brun sable", "#F4A460"),
            ("Orange", "#FFA500"),
            ("Orange foncé", "#FF8C00"),
            ("Chocolat", "#D2691E"),
            ("Brun pérou", "#CD853F"),
            ("Terre de Sienne", "#A0522D"),
            ("Brun cuir", "#8B4513"),
            ("Jaune clair", "#FFFFE0"),
            ("Jaune maïs doux", "#FFF8DC"),
            ("Jaune doré clair", "#FAFAD2"),
            ("Beige citron soie", "#FFFACD"),
            ("Jaune doré pâle", "#EEE8AA"),
            ("Brun kaki", "#F0E68C"),
            ("Jaune", "#FFFF00"),
            ("Or", "#FFD700"),
            ("Jaune doré", "#DAA520"),
            ("Jaune doré foncé", "#B8860B"),
            ("Brun kaki foncé", "#BDB76B"),
            ("Jaune vert", "#9ACD32"),
            ("Kaki", "#6B8E23"),
            ("Olive", "#808000"),
            ("Vert olive foncé", "#556B2F"),
            ("Vert jaune", "#ADFF2F"),
            ("Chartreuse", "#7FFF00"),
            ("Vert prairie", "#7CFC00"),
            ("Citron vert", "#00FF00"),
            ("Citron vert foncé", "#32CD32"),
            ("Blanc menthe", "#F5FFFA"),
            ("Miellat", "#F0FFF0"),
            ("Vert pâle", "#98FB98"),
            ("Vert clair", "#90EE90"),
            ("Vert printemps", "#00FF7F"),
            ("Vert printemps moyen", "#00FA9A"),
            ("Vert forêt", "#228B22"),
            ("Vert", "#008000"),
            ("Vert foncé", "#006400"),
            ("Vert océan foncé", "#8FBC8F"),
            ("Vert océan moyen", "#3CB371"),
            ("Vert océan", "#2E8B57"),
            ("Gris ardoise clair", "#778899"),
            ("Gris ardoise", "#708090"),
            ("Gris ardoise foncé", "#2F4F4F"),
            ("Bleu alice", "#F0F8FF"),
            ("Bleu azur", "#F0FFFF"),
            ("Cyan clair", "#E0FFFF"),
            ("Azurin", "#AFEEEE"),
            ("Aigue-marine", "#7FFFD4"),
            ("Aigue-marine moyen", "#66CDAA"),
            ("Cyan", "#00FFFF"),
            ("Turquoise", "#40E0D0"),
            ("Turquoise moyen", "#48D1CC"),
            ("Turquoise foncé", "#00CED1"),
            ("Vert marin clair", "#20B2AA"),
            ("Cyan foncé", "#008B8B"),
            ("Vert sarcelle", "#008080"),
            ("Bleu pétrole", "#5F9EA0"),
            ("Bleu poudre", "#B0E0E6"),
            ("Bleu clair", "#ADD8E6"),
            ("Bleu azur clair", "#87CEFA"),
            ("Bleu azur", "#87CEEB"),
            ("Bleu azur profond", "#00BFFF"),
            ("Bleu toile", "#1E90FF"),
            ("Bleu lavande", "#E6E6FA"),
            ("Bleu acier clair", "#B0C4DE"),
            ("Bleuet", "#6495ED"),
            ("Bleu acier", "#4682B4"),
            ("Bleu royal", "#4169E1"),
            ("Bleu", "#0000FF"),
            ("Bleu moyen", "#0000CD"),
            ("Bleu foncé", "#00008B"),
            ("Bleu marin", "#000080"),
            ("Bleu de minuit", "#191970"),
        )
    )

    safe_colors = (
        "noir",
        "bordeaux",
        "vert",
        "rouge",
        "violet",
        "sarcelle",
        "bleu",
        "argent",
        "gris",
        "jaune",
        "fuchsia",
        "cyan",
        "blanc",
    )



import re

from unittest.mock import patch

import pytest


class TestCurrencyProvider:
    """Test currency provider methods"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency import Provider as CurrencyProvider

        cls.provider = CurrencyProvider
        cls.currencies = cls.provider.currencies
        cls.cryptocurrencies = cls.provider.cryptocurrencies
        cls.currency_codes, cls.currency_names = tuple(zip(*cls.currencies))
        cls.cryptocurrency_codes, cls.cryptocurrency_names = tuple(zip(*cls.cryptocurrencies))

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert isinstance(cur, tuple)
            assert cur in self.currencies

    def test_currency_code(self, faker, num_samples):
        for _ in range(num_samples):
            code = faker.currency_code()
            assert isinstance(code, str) and code in self.currency_codes

    def test_currency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.currency_name()
            assert isinstance(name, str) and name in self.currency_names

    def test_currency_code_has_symbol(self, faker, num_samples):
        for _ in range(num_samples):
            code = faker.currency_code()
            symbol = faker.currency_symbol(code=code)
            assert isinstance(symbol, str)

    def test_currency_symbol_no_code_supplied(self, faker, num_samples):
        for _ in range(num_samples):
            symbol = faker.currency_symbol()
            assert isinstance(symbol, str)
            assert symbol in self.provider.currency_symbols.values()

    @patch("faker.providers.currency.Provider.random_element")
    def test_currency_symbol_with_valid_code(self, mock_random_element, faker):
        symbol = faker.currency_symbol(code="USD")
        assert symbol == "$"
        mock_random_element.assert_not_called()

    @patch("faker.providers.currency.Provider.random_element")
    def test_currency_symbol_with_invalid_code(self, mock_random_element, faker):
        invalid_code = "FTW"
        with pytest.raises(KeyError):
            faker.currency_symbol(code=invalid_code)
        mock_random_element.assert_not_called()

    def test_cryptocurrency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.cryptocurrency()
            assert isinstance(cur, tuple)
            assert cur in self.cryptocurrencies

    def test_cryptocurrency_code(self, faker, num_samples):
        for _ in range(num_samples):
            code = faker.cryptocurrency_code()
            assert isinstance(code, str) and code in self.cryptocurrency_codes

    def test_cryptocurrency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.cryptocurrency_name()
            assert isinstance(name, str) and name in self.cryptocurrency_names

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestAzAz:
    """Test az_AZ currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.az_AZ import Provider as AzAzCurrencyProvider

        cls.provider = AzAzCurrencyProvider
        cls.currencies = cls.provider.currencies

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert isinstance(cur, tuple) and cur in self.currencies

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestRuRu:
    """Test ru_RU currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.ru_RU import Provider as RuRuCurrencyProvider

        cls.provider = RuRuCurrencyProvider
        cls.currencies = cls.provider.currencies
        cls.currency_codes, cls.currency_names = tuple(zip(*cls.currencies))

        cls.pricetag_pattern = re.compile(r"\d{1,3}(?:\s\d{3})*,\d{2}\sр\.")

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert isinstance(cur, tuple) and cur in self.currencies

    def test_currency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.currency_name()
            assert isinstance(name, str) and name in self.currency_names

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)
            assert self.pricetag_pattern.fullmatch(pricetag)


class TestCsCz:
    """Test cs_CZ currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.cs_CZ import Provider as CsCzCurrencyProvider

        cls.provider = CsCzCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestDeAt:
    """Test de_AT currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.de_AT import Provider as DeAtCurrencyProvider

        cls.provider = DeAtCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestDeDe:
    """Test de_DE currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.de_DE import Provider as DeDeCurrencyProvider

        cls.provider = DeDeCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestEnAu:
    """Test en_AU currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.en_AU import Provider as EnAuCurrencyProvider

        cls.provider = EnAuCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestEnCa:
    """Test en_CA currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.en_CA import Provider as EnCaCurrencyProvider

        cls.provider = EnCaCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestEsEs:
    """Test es_ES currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.es_ES import Provider as EsEsCurrencyProvider

        cls.provider = EsEsCurrencyProvider
        cls.currencies = cls.provider.currencies
        cls.currency_codes, cls.currency_names = tuple(zip(*cls.currencies))

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert cur in self.currencies

    def test_currency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.currency_name()
            assert name in self.currency_names

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestFaIr:
    """Test fa_IR currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.fa_IR import Provider as FaIrCurrencyProvider

        cls.provider = FaIrCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestFrCa:
    """Test fr_CA currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.fr_CA import Provider as FrCaCurrencyProvider

        cls.provider = FrCaCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestFrFr:
    """Test fr_FR currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.fr_FR import Provider as FrFrCurrencyProvider

        cls.provider = FrFrCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestItIt:
    """Test it_IT currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.it_IT import Provider as ItItCurrencyProvider

        cls.provider = ItItCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestPlPl:
    """Test pl_PL currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.pl_PL import Provider as PlPlCurrencyProvider

        cls.provider = PlPlCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestSkSk:
    """Test sk_SK currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.sk_SK import Provider as SkSkCurrencyProvider

        cls.provider = SkSkCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestSvSe:
    """Test sv_SE currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.sv_SE import Provider as SvSeCurrencyProvider

        cls.provider = SvSeCurrencyProvider
        cls.currencies = cls.provider.currencies
        cls.currency_codes, cls.currency_names = tuple(zip(*cls.currencies))

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert cur in self.currencies

    def test_currency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.currency_name()
            assert name in self.currency_names


class TestThTh:
    """Test th_TH currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.th_TH import Provider as ThThCurrencyProvider

        cls.provider = ThThCurrencyProvider
        cls.currencies = cls.provider.currencies
        cls.currency_codes, cls.currency_names = tuple(zip(*cls.currencies))

    def test_currency(self, faker, num_samples):
        for _ in range(num_samples):
            cur = faker.currency()
            assert cur in self.currencies

    def test_currency_name(self, faker, num_samples):
        for _ in range(num_samples):
            name = faker.currency_name()
            assert name in self.currency_names


class TestRoRo:
    """Test ro_RO currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.ro_RO import Provider as RoRoCurrencyProvider

        cls.provider = RoRoCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestPtBr:
    """Test pt_BR currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.pt_BR import Provider as PtBrCurrencyProvider

        cls.provider = PtBrCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestNlNl:
    """Test nl_NL currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.nl_NL import Provider as NlCurrencyProvider

        cls.provider = NlCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestElGr:
    """Test nl_NL currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.el_GR import Provider as ElGrCurrencyProvider

        cls.provider = ElGrCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)


class TestTrTr:
    """Test tr_TR currency provider"""

    num_samples = 100

    @classmethod
    def setup_class(cls):
        from faker.providers.currency.tr_TR import Provider as TrTrCurrencyProvider

        cls.provider = TrTrCurrencyProvider

    def test_pricetag(self, faker, num_samples):
        for _ in range(num_samples):
            pricetag = faker.pricetag()
            assert isinstance(pricetag, str)



import bisect
import itertools

from random import Random
from typing import Generator, Iterable, Optional, Sequence, TypeVar

from faker.generator import random as mod_random


def random_sample(random: Optional[Random] = None) -> float:
    if random is None:
        random = mod_random
    return random.uniform(0.0, 1.0)


def cumsum(it: Iterable[float]) -> Generator[float, None, None]:
    total: float = 0
    for x in it:
        total += x
        yield total


T = TypeVar("T")


def choices_distribution_unique(
    a: Sequence[T],
    p: Optional[Sequence[float]],
    random: Optional[Random] = None,
    length: int = 1,
) -> Sequence[T]:
    # As of Python 3.7, there isn't a way to sample unique elements that takes
    # weight into account.
    if random is None:
        random = mod_random

    assert p is not None
    assert len(a) == len(p)
    assert len(a) >= length, "You can't request more unique samples than elements in the dataset."

    choices = []
    items = list(a)
    probabilities = list(p)
    for i in range(length):
        cdf = tuple(cumsum(probabilities))
        normal = cdf[-1]
        cdf2 = [i / normal for i in cdf]
        uniform_sample = random_sample(random=random)
        idx = bisect.bisect_right(cdf2, uniform_sample)
        item = items[idx]
        choices.append(item)
        probabilities.pop(idx)
        items.pop(idx)
    return choices


def choices_distribution(
    a: Sequence[T],
    p: Optional[Sequence[float]],
    random: Optional[Random] = None,
    length: int = 1,
) -> Sequence[T]:
    if random is None:
        random = mod_random

    if p is not None:
        assert len(a) == len(p)

    if hasattr(random, "choices"):
        if length == 1 and p is None:
            return [random.choice(a)]
        else:
            return random.choices(a, weights=p, k=length)
    else:
        choices = []

        if p is None:
            p = itertools.repeat(1, len(a))  # type: ignore

        cdf = list(cumsum(p))  # type: ignore
        normal = cdf[-1]
        cdf2 = [i / normal for i in cdf]
        for i in range(length):
            uniform_sample = random_sample(random=random)
            idx = bisect.bisect_right(cdf2, uniform_sample)
            item = a[idx]
            choices.append(item)
        return choices



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``it_IT`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Italy
    """

    license_formats = (
        # 1994-present
        "??###??",
    )



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    cellphone_formats = (
        "+994{{provider_code}}{{start_digit}}######",
        "0{{provider_code}} {{start_digit}}## ## ##",
        "0{{provider_code}}-{{start_digit}}##-##-##",
    )

    telephone_formats = ("{{area_code}}## ## ##",)

    provider_codes = ["50", "51", "55", "60", "70", "77", "99"]

    start_digits = ["2", "3", "4", "5", "6", "7", "8", "9"]

    area_codes = [
        "012 3",
        "012 4",
        "012 5",
        "018 6",
        "020 2",
        "021 2",
        "021 4",
        "022 2",
        "022 3",
        "022 4",
        "023 3",
        "024 2",
        "025 2",
        "026 2",
        "026 3",
        "036 5",
    ]

    formats = cellphone_formats + telephone_formats

    def start_digit(self) -> str:
        return self.random_element(self.start_digits)

    def provider_code(self) -> str:
        return self.random_element(self.provider_codes)

    def area_code(self) -> str:
        return self.random_element(self.area_codes)

    def cellphone_number(self) -> str:
        pattern: str = self.random_element(self.cellphone_formats)
        return self.numerify(self.generator.parse(pattern))

    def landline_number(self) -> str:
        pattern: str = self.random_element(self.telephone_formats)
        return self.numerify(self.generator.parse(pattern))

    def phone_number(self) -> str:
        pattern: str = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



from enum import Enum, auto

import pytest

from faker.providers.python import EmptyEnumException


class _TestEnumWithNoElements(Enum):
    pass


class _TestEnumWithSingleElement(Enum):
    Single = auto


class _TestEnum(Enum):
    A = auto
    B = auto
    C = auto


class TestEnumProvider:
    num_samples = 100

    def test_enum(self, faker, num_samples):
        # (1/3) ** 100 ~ 1.94e-48 probability of this test failing because a specific
        # value was not sampled
        for _ in range(num_samples):
            actual = faker.enum(_TestEnum)
            assert actual in (_TestEnum.A, _TestEnum.B, _TestEnum.C)

    def test_enum_single(self, faker):
        assert faker.enum(_TestEnumWithSingleElement) == _TestEnumWithSingleElement.Single
        assert faker.enum(_TestEnumWithSingleElement) == _TestEnumWithSingleElement.Single

    def test_empty_enum_raises(self, faker):
        with pytest.raises(
            EmptyEnumException,
            match="The provided Enum: '_TestEnumWithNoElements' has no members.",
        ):
            faker.enum(_TestEnumWithNoElements)

    def test_none_raises(self, faker):
        with pytest.raises(ValueError):
            faker.enum(None)

    def test_incorrect_type_raises(self, faker):
        not_an_enum_type = type("NotAnEnumType")
        with pytest.raises(TypeError):
            faker.enum(not_an_enum_type)



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    """
    Implement internet provider for ``bn_BD`` locale.
    """

    free_email_domains = (
        "gmail.com",
        "yahoo.com",
        "hotmail.com",
        "mail.ru",
        "yandex.ru",
        "rambler.ru",
    )

    tlds = (
        "com",
        "com",
        "com",
        "com",
        "com",
        "com",
        "biz",
        "info",
        "net",
        "org",
        "com.bd",
    )



from ..de import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ("{{city_name}}",)

    city_with_postcode_formats = ("{{postcode}} {{city}}",)

    street_name_formats = (
        "{{first_name}}-{{last_name}}-{{street_suffix_long}}",
        "{{last_name}}{{street_suffix_short}}",
    )
    street_address_formats = ("{{street_name}} {{building_number}}",)
    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = ("###", "##", "#", "#/#")

    street_suffixes_long = (
        "Gasse",
        "Platz",
        "Ring",
        "Straße",
        "Weg",
        "Allee",
    )
    street_suffixes_short = (
        "gasse",
        "platz",
        "ring",
        "straße",
        "str.",
        "weg",
        "allee",
    )

    postcode_formats = ("#####",)

    cities = (
        "Aachen",
        "Ahaus",
        "Altentreptow",
        "Altötting",
        "Amberg",
        "Angermünde",
        "Anklam",
        "Ansbach",
        "Apolda",
        "Arnstadt",
        "Artern",
        "Aschaffenburg",
        "Aue",
        "Auerbach",
        "Augsburg",
        "Aurich",
        "Backnang",
        "Bad Brückenau",
        "Bad Freienwalde",
        "Bad Kissingen",
        "Bad Kreuznach",
        "Bad Langensalza",
        "Bad Liebenwerda",
        "Bad Mergentheim",
        "Badalzungen",
        "Badibling",
        "Badoberan",
        "Bamberg",
        "Bautzen",
        "Bayreuth",
        "Beeskow",
        "Beilngries",
        "Belzig",
        "Berchtesgaden",
        "Bergzabern",
        "Berlin",
        "Bernburg",
        "Bersenbrück",
        "Biedenkopf",
        "Bischofswerda",
        "Bitterfeld",
        "Bogen",
        "Borken",
        "Borna",
        "Brand",
        "Brandenburg",
        "Bremen",
        "Bremervörde",
        "Brilon",
        "Bruchsal",
        "Burg",
        "Burgdorf",
        "Burglengenfeld",
        "Böblingen",
        "Büsingen am Hochrhein",
        "Bützow",
        "Calau",
        "Calw",
        "Celle",
        "Chemnitz",
        "Cloppenburg",
        "Coburg",
        "Cottbus",
        "Crailsheim",
        "Cuxhaven",
        "Dachau",
        "Darmstadt",
        "Deggendorf",
        "Delitzsch",
        "Demmin",
        "Dessau",
        "Dieburg",
        "Diepholz",
        "Dinkelsbühl",
        "Dinslaken",
        "Donaueschingen",
        "Dresden",
        "Duderstadt",
        "Döbeln",
        "Düren",
        "Ebermannstadt",
        "Ebern",
        "Ebersberg",
        "Eberswalde",
        "Eckernförde",
        "Eggenfelden",
        "Eichstätt",
        "Eichstätt",
        "Eilenburg",
        "Einbeck",
        "Eisenach",
        "Eisenberg",
        "Eisenhüttenstadt",
        "Eisleben",
        "Emmendingen",
        "Erbisdorf",
        "Erding",
        "Erfurt",
        "Erkelenz",
        "Euskirchen",
        "Eutin",
        "Fallingbostel",
        "Feuchtwangen",
        "Finsterwalde",
        "Flöha",
        "Forchheim",
        "Forst",
        "Freising",
        "Freital",
        "Freudenstadt",
        "Fulda",
        "Fürstenfeldbruck",
        "Fürstenwalde",
        "Füssen",
        "Gadebusch",
        "Gardelegen",
        "Garmisch-Partenkirchen",
        "Geithain",
        "Geldern",
        "Gelnhausen",
        "Genthin",
        "Gera",
        "Germersheim",
        "Gerolzhofen",
        "Gießen",
        "Gifhorn",
        "Goslar",
        "Gotha",
        "Grafenau",
        "Gransee",
        "Greifswald",
        "Greiz",
        "Grevenbroich",
        "Grevesmühlen",
        "Griesbach Rottal",
        "Grimma",
        "Grimmen",
        "Groß-Gerau",
        "Großenhain",
        "Gräfenhainichen",
        "Guben",
        "Gunzenhausen",
        "Göppingen",
        "Görlitz",
        "Göttingen",
        "Günzburg",
        "Güstrow",
        "Gütersloh",
        "Hagenow",
        "Hainichen",
        "Halberstadt",
        "Haldensleben",
        "Hamburg",
        "Hammelburg",
        "Hannover",
        "Hannoversch Münden",
        "Hansestadttralsund",
        "Havelberg",
        "Hechingen",
        "Heiligenstadt",
        "Heinsberg",
        "Helmstedt",
        "Herford",
        "Hersbruck",
        "Herzberg",
        "Hettstedt",
        "Hildburghausen",
        "Hildesheim",
        "Hofgeismar",
        "Hohenmölsen",
        "Hohenstein-Ernstthal",
        "Holzminden",
        "Hoyerswerda",
        "Husum",
        "Höxter",
        "Hünfeld",
        "Illertissen",
        "Ilmenau",
        "Ingolstadt",
        "Iserlohn",
        "Jena",
        "Jessen",
        "Jülich",
        "Jüterbog",
        "Kaiserslautern",
        "Kamenz",
        "Karlsruhe",
        "Kassel",
        "Kehl",
        "Kelheim",
        "Kemnath",
        "Kitzingen",
        "Kleve",
        "Klötze",
        "Koblenz",
        "Konstanz",
        "Kronach",
        "Kulmbach",
        "Kusel",
        "Kyritz",
        "Königs Wusterhausen",
        "Kötzting",
        "Leipziger Land",
        "Lemgo",
        "Lichtenfels",
        "Lippstadt",
        "Lobenstein",
        "Luckau",
        "Luckenwalde",
        "Ludwigsburg",
        "Ludwigslust",
        "Lörrach",
        "Lübben",
        "Lübeck",
        "Lübz",
        "Lüdenscheid",
        "Lüdinghausen",
        "Lüneburg",
        "Magdeburg",
        "Main-Höchst",
        "Mainburg",
        "Malchin",
        "Mallersdorf",
        "Marienberg",
        "Marktheidenfeld",
        "Mayen",
        "Meiningen",
        "Meißen",
        "Melle",
        "Mellrichstadt",
        "Melsungen",
        "Meppen",
        "Merseburg",
        "Mettmann",
        "Miesbach",
        "Miltenberg",
        "Mittweida",
        "Moers",
        "Monschau",
        "Mühldorf am Inn",
        "Mühlhausen",
        "München",
        "Nabburg",
        "Naila",
        "Nauen",
        "Neu-Ulm",
        "Neubrandenburg",
        "Neunburg vorm Wald",
        "Neuruppin",
        "Neuss",
        "Neustadt am Rübenberge",
        "Neustadtner Waldnaab",
        "Neustrelitz",
        "Niesky",
        "Norden",
        "Nordhausen",
        "Northeim",
        "Nördlingen",
        "Nürtingen",
        "Oberviechtach",
        "Ochsenfurt",
        "Olpe",
        "Oranienburg",
        "Oschatz",
        "Osterburg",
        "Osterode am Harz",
        "Paderborn",
        "Parchim",
        "Parsberg",
        "Pasewalk",
        "Passau",
        "Pegnitz",
        "Peine",
        "Perleberg",
        "Pfaffenhofen an der Ilm",
        "Pinneberg",
        "Pirmasens",
        "Plauen",
        "Potsdam",
        "Prenzlau",
        "Pritzwalk",
        "Pößneck",
        "Quedlinburg",
        "Querfurt",
        "Rastatt",
        "Rathenow",
        "Ravensburg",
        "Recklinghausen",
        "Regen",
        "Regensburg",
        "Rehau",
        "Reutlingen",
        "Ribnitz-Damgarten",
        "Riesa",
        "Rochlitz",
        "Rockenhausen",
        "Roding",
        "Rosenheim",
        "Rostock",
        "Roth",
        "Rothenburg ob der Tauber",
        "Rottweil",
        "Rudolstadt",
        "Saarbrücken",
        "Saarlouis",
        "Sangerhausen",
        "Sankt Goar",
        "Sankt Goarshausen",
        "Saulgau",
        "Scheinfeld",
        "Schleiz",
        "Schlüchtern",
        "Schmölln",
        "Schongau",
        "Schrobenhausen",
        "Schwabmünchen",
        "Schwandorf",
        "Schwarzenberg",
        "Schweinfurt",
        "Schwerin",
        "Schwäbisch Gmünd",
        "Schwäbisch Hall",
        "Sebnitz",
        "Seelow",
        "Senftenberg",
        "Siegen",
        "Sigmaringen",
        "Soest",
        "Soltau",
        "Soltau",
        "Sondershausen",
        "Sonneberg",
        "Spremberg",
        "Stade",
        "Stade",
        "Stadtroda",
        "Stadtsteinach",
        "Staffelstein",
        "Starnberg",
        "Staßfurt",
        "Steinfurt",
        "Stendal",
        "Sternberg",
        "Stollberg",
        "Strasburg",
        "Strausberg",
        "Stuttgart",
        "Suhl",
        "Sulzbach-Rosenberg",
        "Säckingen",
        "Sömmerda",
        "Tecklenburg",
        "Teterow",
        "Tirschenreuth",
        "Torgau",
        "Tuttlingen",
        "Tübingen",
        "Ueckermünde",
        "Uelzen",
        "Uffenheim",
        "Vechta",
        "Viechtach",
        "Viersen",
        "Vilsbiburg",
        "Vohenstrauß",
        "Waldmünchen",
        "Wanzleben",
        "Waren",
        "Warendorf",
        "Weimar",
        "Weißenfels",
        "Weißwasser",
        "Werdau",
        "Wernigerode",
        "Wertingen",
        "Wesel",
        "Wetzlar",
        "Wiedenbrück",
        "Wismar",
        "Wittenberg",
        "Wittmund",
        "Wittstock",
        "Witzenhausen",
        "Wolfach",
        "Wolfenbüttel",
        "Wolfratshausen",
        "Wolgast",
        "Wolmirstedt",
        "Worbis",
        "Wunsiedel",
        "Wurzen",
        "Zerbst",
        "Zeulenroda",
        "Zossen",
        "Zschopau",
    )

    states = (
        "Baden-Württemberg",
        "Bayern",
        "Berlin",
        "Brandenburg",
        "Bremen",
        "Hamburg",
        "Hessen",
        "Mecklenburg-Vorpommern",
        "Niedersachsen",
        "Nordrhein-Westfalen",
        "Rheinland-Pfalz",
        "Saarland",
        "Sachsen",
        "Sachsen-Anhalt",
        "Schleswig-Holstein",
        "Thüringen",
    )

    def street_suffix_short(self) -> str:
        return self.random_element(self.street_suffixes_short)

    def street_suffix_long(self) -> str:
        return self.random_element(self.street_suffixes_long)

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit

    def city_with_postcode(self) -> str:
        pattern: str = self.random_element(self.city_with_postcode_formats)
        return self.generator.parse(pattern)



import re

from typing import Pattern

from faker.providers.bank.ru_RU import Provider as RuRuBankProvider
from faker.providers.bank.uk_UA import Provider as UkUaBankProvider
from faker.providers.credit_card import Provider as CreditCardProvider


class TestCreditCardProvider:
    """Test credit card provider methods"""

    mastercard_pattern: Pattern = re.compile(
        r"(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}",
    )
    visa_pattern: Pattern = re.compile(r"4[0-9]{12}([0-9]{3}){0,2}")
    discover_pattern: Pattern = re.compile(r"6(?:011|5[0-9]{2})[0-9]{12}")
    diners_club_pattern: Pattern = re.compile(r"3(?:0[0-5]|[68][0-9])[0-9]{11}")
    jcb_pattern: Pattern = re.compile(r"(?:2131|1800|35\d{3})\d{11}")

    def test_mastercard(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_mastercard:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 16)
                assert len(number) == 16
                assert self.mastercard_pattern.fullmatch(number)

    def test_visa13(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_visa:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 13)
                assert len(number) == 13
                assert self.visa_pattern.fullmatch(number)

    def test_visa16(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_visa:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 16)
                assert len(number) == 16
                assert self.visa_pattern.fullmatch(number)

    def test_visa19(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_visa:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 19)
                assert len(number) == 19
                assert self.visa_pattern.fullmatch(number)

    def test_discover(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_discover:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 16)
                assert len(number) == 16
                assert self.discover_pattern.fullmatch(number)

    def test_diners_club(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_diners:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 14)
                assert len(number) == 14
                assert self.diners_club_pattern.fullmatch(number)

    def test_jcb16(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_jcb16:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 16)
                assert len(number) == 16
                assert self.jcb_pattern.fullmatch(number)

    def test_jcb15(self, faker, num_samples):
        provider = CreditCardProvider(faker)
        for prefix in provider.prefix_jcb15:
            for _ in range(num_samples):
                number = provider._generate_number(prefix, 15)
                assert len(number) == 15
                assert self.jcb_pattern.fullmatch(number)


class TestRuRu:
    """Test ru_RU credit card provider methods"""

    visa_pattern: Pattern = re.compile(r"4[0-9]{15}")
    mastercard_pattern: Pattern = re.compile(
        r"(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}",
    )
    mir_pattern: Pattern = re.compile(r"220[0-4][0-9]{12}")
    maestro_pattern: Pattern = re.compile(r"(?:50|5[6-9]|6[0-9])[0-9]{14}")
    amex_pattern: Pattern = re.compile(r"3[4|7][0-9]{13}")
    unionpay_pattern: Pattern = re.compile(r"(?:62|81)[0-9]{14}")

    def test_visa(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("visa")
            assert self.visa_pattern.fullmatch(number)

    def test_mastercard(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("mastercard")
            assert self.mastercard_pattern.fullmatch(number)

    def test_mir(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("mir")
            assert self.mir_pattern.fullmatch(number)

    def test_maestro(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("maestro")
            assert self.maestro_pattern.fullmatch(number)

    def test_amex(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("amex")
            assert self.amex_pattern.fullmatch(number)

    def test_unionpay(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("unionpay")
            assert self.unionpay_pattern.fullmatch(number)

    def test_credit_card_full(self, faker, num_samples):
        for _ in range(num_samples):
            card_data = faker.credit_card_full().split("\n")
            assert re.match("[A-Za-z]+", card_data[1])
            assert card_data[4] in RuRuBankProvider.banks


class TestPtPt:
    """Test pt_PT credit card provider methods"""

    visa_pattern: Pattern = re.compile(r"4[0-9]{15}")
    mastercard_pattern: Pattern = re.compile(r"5[1-5][0-9]{14}")
    maestro_pattern: Pattern = re.compile(r"(50|67)[0-9]{14}")

    def test_visa(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("visa")
            assert self.visa_pattern.fullmatch(number)

    def test_mastercard(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("mastercard")
            assert self.mastercard_pattern.fullmatch(number)

    def test_maestro(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("maestro")
            assert self.maestro_pattern.fullmatch(number)


class TestUkUa:
    mastercard_pattern: Pattern = re.compile(
        r"(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}",
    )
    visa_pattern: Pattern = re.compile(r"4[0-9]{12}([0-9]{3}){0,2}")
    maestro_pattern: Pattern = re.compile(r"(67)[0-9]{14}")
    prostir_pattern: Pattern = re.compile(r"(9)[0-9]{15}")

    def test_mastercard(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("mastercard")
            assert self.mastercard_pattern.fullmatch(number)

    def test_visa(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("visa")
            assert self.visa_pattern.fullmatch(number)

    def test_maestro(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("maestro")
            assert self.maestro_pattern.fullmatch(number)

    def test_prostir(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.credit_card_number("prostir")
            assert self.prostir_pattern.fullmatch(number)

    def test_credit_card_full(self, faker, num_samples):
        for _ in range(num_samples):
            card_data = faker.credit_card_full("prostir").split("\n")
            assert re.match("[A-Za-z]+", card_data[1])
            assert card_data[4] in UkUaBankProvider.banks
            assert card_data[0] == "ПРОСТІР"



import re
import unittest

from faker import Faker


class TestFile(unittest.TestCase):
    """Tests file"""

    def setUp(self):
        self.fake = Faker()
        Faker.seed(0)

    def test_file_name(self):
        for _ in range(100):
            file_name = self.fake.file_name()
            assert re.search(r"\w+\.\w+", file_name)
            file_name = self.fake.file_name(extension=None)
            assert re.search(r"\w+\.\w+", file_name)
            file_name = self.fake.file_name(extension="pdf")
            assert re.search(r"\w+\.pdf$", file_name)
            file_name = self.fake.file_name(category="image")
            assert re.search(r"\w+\.(bmp|gif|jpeg|jpg|png|tiff)$", file_name)
            file_name = self.fake.file_name(category="image", extension="abcdef")
            assert re.search(r"\w+\.abcdef$", file_name)
            file_name = self.fake.file_name(extension="")
            assert re.search(r"\w+$", file_name)

    def test_file_path(self):
        for _ in range(100):
            file_path = self.fake.file_path()
            assert re.search(r"\/\w+\/\w+\.\w+", file_path)
            file_path = self.fake.file_path(absolute=False)
            assert re.search(r"\w+\/\w+\.\w+", file_path)
            file_path = self.fake.file_path(depth=3)
            assert re.search(r"\/\w+\/\w+\/\w+\.\w+", file_path)
            file_path = self.fake.file_path(extension="pdf")
            assert re.search(r"\/\w+\/\w+\.pdf$", file_path)
            file_path = self.fake.file_path(extension=["a", "bc", "def", "ghij", "klmno"])
            assert re.search(r"\/\w+\/\w+\.(a|bc|def|ghij|klmno)$", file_path)
            file_path = self.fake.file_path(extension=None)
            assert re.search(r"\/\w+\/\w+\.\w+", file_path)
            file_path = self.fake.file_path(extension="")
            assert re.search(r"\/\w+\/\w+$", file_path)
            file_path = self.fake.file_path(extension=[])
            assert re.search(r"\/\w+\/\w+$", file_path)
            file_path = self.fake.file_path(category="image")
            assert re.search(r"\/\w+\/\w+\.(bmp|gif|jpeg|jpg|png|tiff)", file_path)
            file_path = self.fake.file_path(file_system_rule="windows")
            assert re.search(r"\\\w+\\\w+\.\w+", file_path)
            file_path = self.fake.file_path(file_system_rule="windows", category="image", absolute=True)
            assert re.search(r"^[a-zA-Z]:\\\w+\\\w+\.\w+", file_path)
            assert re.search(r"\\\w+\\\w+\.(bmp|gif|jpeg|jpg|png|tiff)$", file_path)

    def test_unix_device(self):
        reg_device = re.compile(r"^/dev/(vd|sd|xvd)[a-z]$")
        # Test default
        for _ in range(100):
            path = self.fake.unix_device()
            assert reg_device.match(path)
        # Test with prefix
        for _ in range(100):
            path = self.fake.unix_device("sd")
            assert reg_device.match(path)
            assert path.startswith("/dev/sd")

    def test_unix_partition(self):
        reg_part = re.compile(r"^/dev/(vd|sd|xvd)[a-z]\d$")
        # Test default
        for _ in range(100):
            path = self.fake.unix_partition()
            assert reg_part.match(path)
        # Test with prefix
        for _ in range(100):
            path = self.fake.unix_partition("sd")
            assert reg_part.match(path)
            assert path.startswith("/dev/sd")



import pytest

from faker.providers.sbn import SBN9
from faker.providers.sbn.en_US import Provider as SBNProvider
from faker.providers.sbn.rules import RegistrantRule


class TestISBN9:
    def test_check_digit_is_correct(self):
        sbn = SBN9(registrant="340", publication="01381")
        assert sbn.check_digit == "X"
        sbn = SBN9(registrant="06", publication="230125")
        assert sbn.check_digit == "2"
        sbn = SBN9(registrant="10103", publication="202")
        assert sbn.check_digit == "3"

    def test_format_length(self):
        sbn = SBN9(registrant="4516", publication="7331")
        assert len(sbn.format()) == 9
        sbn = SBN9(registrant="451", publication="10036")
        assert len(sbn.format()) == 9


class TestProvider:
    prov = SBNProvider(None)

    def test_reg_pub_separation(self):
        r1 = RegistrantRule("0000000", "0000001", 1)
        r2 = RegistrantRule("0000002", "0000003", 2)
        assert self.prov._registrant_publication("00000000", [r1, r2]) == (
            "0",
            "0000000",
        )
        assert self.prov._registrant_publication("00000010", [r1, r2]) == (
            "0",
            "0000010",
        )
        assert self.prov._registrant_publication("00000019", [r1, r2]) == (
            "0",
            "0000019",
        )
        assert self.prov._registrant_publication("00000020", [r1, r2]) == (
            "00",
            "000020",
        )
        assert self.prov._registrant_publication("00000030", [r1, r2]) == (
            "00",
            "000030",
        )
        assert self.prov._registrant_publication("00000031", [r1, r2]) == (
            "00",
            "000031",
        )
        assert self.prov._registrant_publication("00000039", [r1, r2]) == (
            "00",
            "000039",
        )

    def test_rule_not_found(self):
        with pytest.raises(Exception):
            r = RegistrantRule("0000000", "0000001", 1)
            self.prov._registrant_publication("0000002", [r])



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    price_formats = ["#.##", "%#.##", "%##.##", "%,###.##", "%#,###.##"]

    def pricetag(self) -> str:
        return f"{self.numerify(self.random_element(self.price_formats))} ₺"
